#!/usr/bin/perl
#
# $Id: mkconfig.pl,v 1.86 2010-12-30 17:06:42-08 bll Exp $
#
# Copyright 2006-2010 Brad Lanam Walnut Creek, CA USA
#

# HP-UX doesn't have these installed.
# use strict;
# use Config;
require 5.005;

my $CONFH;
my $LOG = "mkconfig.log";
my $TMP = "_tmp_mkconfig";
my $OPTIONFILE = "options.dat";
my $CACHEFILE = "mkconfig.cache";
my $VARSFILE = "mkconfig_c.vars";
my $REQLIB = "mkconfig.reqlibs";

my $precc = <<'_HERE_';
#if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
# define _(x) x
#else
# define _(x) ()
# define void char
#endif
#if defined(__cplusplus) || defined (c_plusplus)
# define CPP_EXTERNS_BEG extern "C" {
# define CPP_EXTERNS_END }
CPP_EXTERNS_BEG
extern int printf (const char *, ...);
CPP_EXTERNS_END
#else
# define CPP_EXTERNS_BEG
# define CPP_EXTERNS_END
#endif
_HERE_

my $optionsloaded = 0;
my %optionshash;
my $iflevels = '';

sub
exitmkconfig
{
    my $rc = shift;
    exit 1;
}

sub
printlabel
{
    my ($name, $label) = @_;
    print LOGFH "## ${iflevels}[$name] $label ... \n";
    print STDOUT "${iflevels}$label ... ";
}

sub
printyesno_actual
{
    my ($name, $val, $tag) = @_;

    print LOGFH "## [$name] $val $tag\n";
    print STDOUT "$val $tag\n";
}

sub
printyesno_val
{
  my ($name, $val, $tag) = @_;

  if ($val ne '0') {
    printyesno_actual ($name, $val, $tag);
  } else {
    printyesno_actual ($name, 'no', $tag);
  }
}

sub
printyesno
{
    my ($name, $val, $tag) = @_;

    if ($val ne "0")
    {
        $val = "yes";
    }
    printyesno_val $name, $val, $tag;
}

sub
savecache
{
    my ($r_clist, $r_config) = @_;

    open (MKCC, ">$CACHEFILE");
    open (MKCV, ">$VARSFILE");
    foreach my $val (@{$r_clist->{'list'}})
    {
      if ($val =~ /^lib__lib_/o) {
        $tval = $val;
        $tval =~ s/^lib_//o;
        print MKCC "di_c_lib_${tval}='" . $r_config->{$val} . "'\n";
      } else {
        print MKCC "di_c_${val}='" . $r_config->{$val} . "'\n";
        print MKCV $val, "\n";
      }
    }
    close (MKCC);
    close (MKCV);
}

sub
checkcache_val
{
  my ($name, $r_config) = @_;

  my $rc = 1;
  if (defined ($r_config->{$name}) && $r_config->{$name} ne "" )
  {
    printyesno_val $name, $r_config->{$name}, " (cached)";
    $rc = 0;
  }
  return $rc;
}

sub
checkcache
{
  my ($name, $r_config) = @_;

  my $rc = 1;
  if (defined ($r_config->{$name}) && $r_config->{$name} ne "" )
  {
    printyesno $name, $r_config->{$name}, " (cached)";
    $rc = 0;
  }
  return $rc;
}

sub
loadoptions
{
  if ($optionsloaded == 0 && open (OPTS, "<$OPTIONFILE")) {
    while (my $o = <OPTS>) {
      chomp $o;
      if ($o =~ /^$/o || $o =~ /^#/o) {
        next;
      }
      my ($onm, $val) = split (/=/, $o);
      printf LOGFH "## load: $onm = $val\n";
      $optionshash{$onm} = $val;
    }
    $optionsloaded = 1;
    close (OPTS);
  }
}

sub
setlist
{
    my ($r_clist, $name) = @_;
    my $r_hash = $r_clist->{'hash'};
    if (! defined ($r_hash->{$name}))
    {
      push @{$r_clist->{'list'}}, $name;
      $r_hash->{$name} = 1;
    }
}

sub
print_headers
{
    my ($r_a, $r_clist, $r_config) = @_;
    my $txt;

    $txt = '';

    if ($r_a->{'incheaders'} eq 'all' ||
        $r_a->{'incheaders'} eq 'std')
    {
        # always include these four if present ...
        foreach my $val ('_hdr_stdio', '_hdr_stdlib', '_sys_types', '_sys_param')
        {
            if (defined ($r_config->{$val}) &&
                 $r_config->{$val} ne '0')
            {
                $txt .= "#include <" . $r_config->{$val} . ">\n";
            }
        }
    }

    if ($r_a->{'incheaders'} eq 'all')
    {
        foreach my $val (@{$r_clist->{'list'}})
        {
            if ($val !~ m#^(_hdr_|_sys_)#o)
            {
                next;
            }
            if ($val eq '_hdr_stdio' ||
                $val eq '_hdr_stdlib' ||
                $val eq '_sys_types' ||
                $val eq '_sys_param')
            {
                next;
            }
            if ($val eq '_sys_time' &&
                $r_config->{'_sys_time'} ne '0' &&
                $r_config->{'_inc_conflict__hdr_time__sys_time'} eq '0')
            {
                next;
            }
            if ($val eq '_hdr_linux_quota' &&
                $r_config->{'_hdr_linux_quota'} ne '0' &&
                $r_config->{'_inc_conflict__sys_quota__hdr_linux_quota'} eq '0')
            {
                next;
            }
            if ($r_config->{$val} ne '0')
            {
                $txt .= "#include <" . $r_config->{$val} . ">\n";
            }
        }
        $txt .= "\n";
    }

    return $txt;
}

sub
_chk_run
{
    my ($name, $code, $r_val, $r_clist, $r_config, $r_a) = @_;

    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', %$r_a, });
    print LOGFH "##  run test: link: $rc\n";
    $$r_val = 0;
    if ($rc == 0)
    {
        $rc = system ("./$name.exe > $name.out");
        if ($rc & 127) { exitmkconfig ($rc); }
        $rc >>= 8;
        print LOGFH "##  run test: run: $rc\n";
        if ($rc == 0)
        {
            open (CRFH, "<$name.out");
            $$r_val = <CRFH>;
            chomp $$r_val;
            close CRFH;
        }
    }
    return $rc;
}

sub
_chk_link
{
    my ($name, $code, $r_clist, $r_config, $r_a) = @_;

    my $otherlibs = '';
    if (defined ($r_a->{'otherlibs'}))
    {
        $otherlibs = $r_a->{'otherlibs'};
    }

    open (CLFH, ">$name.c");
    print CLFH $precc;

    my $hdrs = print_headers ($r_a, $r_clist, $r_config);
    print CLFH $hdrs;
    print CLFH $code;
    close CLFH;

    my $rc = system ("cat $name.c >> $LOG");
    if ($rc & 127) { exitmkconfig ($rc); }

    my $dlibs = '';
    $rc = _chk_link_libs ($name, {} );
    if ($rc != 0)
    {
      if ($otherlibs ne '')
      {
        my @olibs = split (/,/, $otherlibs);
        my $oliblist = '';
        foreach my $olib (@olibs)
        {
          $oliblist = $oliblist . ' ' . $olib;
          $rc = _chk_link_libs ($name, { 'otherlibs' => $oliblist, } );
          if ($rc == 0)
          {

              my $r_hash = $r_config->{'reqlibs'};
              my @vals = split (/\s+/, $oliblist);
              $dlibs = '';
              foreach my $val (@vals)
              {
                  if ($val eq '') { next; }
                  if (! defined ($r_hash->{$val}))
                  {
                      print LOGFH "   reqlib: $val: new\n";
                      push @{$r_config->{'reqlibs_list'}}, $val;
                  } else {
                      print LOGFH "   reqlib: $val: already\n";
                  }
                  $r_hash->{$val} = 1;
                  $dlibs .= $val . ' ';
              }
              last;
          }
        }
      }
    }

    $r_a->{'dlibs'} = $dlibs;

    return $rc;
}

sub
_chk_link_libs
{
    my ($name, $r_a) = @_;

    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} $ENV{'CPPFLAGS'} ";
    if (defined ($r_a->{'cflags'}))
    {
        $cmd .= ' ' . $r_a->{'cflags'} . ' ';
    }
    $cmd .= "-o $name.exe $name.c";
    $cmd .= " $ENV{'LDFLAGS'} $ENV{'LIBS'}";
    if (defined ($r_a->{'otherlibs'}) && $r_a->{'otherlibs'} ne undef)
    {
        $cmd .= ' ' . $r_a->{'otherlibs'} . ' ';
    }
    print LOGFH "##  link test: $cmd\n";
    my $rc = system ("$cmd >> $LOG 2>&1");
    if ($rc & 127) { exitmkconfig ($rc); }
    print LOGFH "##      link test: $rc\n";
    if ($rc == 0)
    {
        if (! -x "$name.exe")  # not executable.
        {
            $rc = 1;
        }
    }
    return $rc;
}

sub
_check_cpp
{
    my ($name, $code, $r_a, $r_clist, $r_config) = @_;

    open (CCFH, ">$name.c");
    print CCFH $precc;
    print CCFH $code;
    close CCFH;

    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} $ENV{'CPPFLAGS'} ";
    if (defined ($r_a->{'cflags'}))
    {
        $cmd .= ' ' . $r_a->{'cflags'} . ' ';
    }
    $cmd .= "-E $name.c > $name.out ";
    my $rc = system ("cat $name.c >> $LOG");
    if ($rc & 127) { exitmkconfig ($rc); }
    $rc = system ("$cmd 2>>$LOG");
    print LOGFH "##  cpp test: $cmd\n";
    if ($rc & 127) { exitmkconfig ($rc); }
    print LOGFH "##      cpp test: $rc\n";
    if ($rc == 0 && ! -f "$name.out") {
      print LOGFH "##      can't locate $name.out\n";
      $rc = -2;
    }
    return $rc;
}

sub
_chk_compile
{
    my ($name, $code, $r_clist, $r_config, $r_a) = @_;

    open (CCFH, ">$name.c");

    print CCFH $precc;

    my $hdrs = print_headers ($r_a, $r_clist, $r_config);
    print CCFH $hdrs;
    print CCFH $code;
    close CCFH;

    my $cmd = "$ENV{'CC'} $ENV{'CFLAGS'} $ENV{'CPPFLAGS'} -c $name.c";
    print LOGFH "##  compile test: $cmd\n";
    my $rc = system ("cat $name.c >> $LOG");
    if ($rc & 127) { exitmkconfig ($rc); }
    $rc = system ("$cmd >> $LOG 2>&1");
    if ($rc & 127) { exitmkconfig ($rc); }
    print LOGFH "##  compile test: $rc\n";
    return $rc;
}

sub
do_chk_compile
{
    my ($name, $code, $inc, $r_clist, $r_config) = @_;

    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
        { 'incheaders' => $inc, });
    my $trc = 0;
    if ($rc == 0)
    {
        $trc = 1;
    }
    printyesno $name, $trc;
    setlist $r_clist, $name;
    $r_config->{$name} = $trc;
}

sub
check_header
{
    my ($name, $file, $r_clist, $r_config, $r_a) = @_;

    printlabel $name, "header: $file";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    my $r_rh = $r_a->{'reqhdr'} || [];
    my $code = '';
    foreach my $reqhdr (@$r_rh)
    {
        $code .= <<"_HERE_";
#include <$reqhdr>
_HERE_
    }
    $code .= <<"_HERE_";
#include <${file}>
main () { return (0); }
_HERE_
    my $rc = 1;
    $rc = _chk_compile ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'std', });
    my $val = 0;
    if ($rc == 0)
    {
        $val = $file;
    }
    printyesno $name, $val;
    setlist $r_clist, $name;
    $r_config->{$name} = $val;
}

sub
check_constant
{
    my ($name, $constant, $r_clist, $r_config, $r_a) = @_;

    printlabel $name, "constant: $constant";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    my $r_rh = $r_a->{'reqhdr'} || [];
    my $code = '';
    foreach my $reqhdr (@$r_rh)
    {
        $code .= <<"_HERE_";
#include <$reqhdr>
_HERE_
    }
    $code .= <<"_HERE_";
main () { if (${constant} == 0) { 1; } return (0); }
_HERE_
    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
}

# if the keyword is reserved, the compile will fail.
sub
check_keyword
{
    my ($name, $keyword, $r_clist, $r_config) = @_;

    printlabel $name, "keyword: $keyword";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    $r_config->{$name} = 0;
    my $code = <<"_HERE_";
main () { int ${keyword}; ${keyword} = 1; return (0); }
_HERE_
    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'std', });
    setlist $r_clist, $name;
    if ($rc != 0)  # failure means it is reserved...
    {
        $r_config->{$name} = 1;
    }
    printyesno $name, $r_config->{$name};
}

sub
check_proto
{
    my ($name, $r_clist, $r_config) = @_;

    printlabel $name, "supported: prototypes";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    my $code = <<"_HERE_";
CPP_EXTERNS_BEG
extern int foo (int, int);
CPP_EXTERNS_END
int bar () { int rc; rc = foo (1,1); return 0; }
_HERE_
    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', });
    setlist $r_clist, $name;
    $r_config->{$name} = 0;
    if ($rc == 0)
    {
        $r_config->{$name} = 1;
    }
    printyesno $name, $r_config->{$name};
}

sub
check_command
{
    my ($name, $cmd, $r_clist, $r_config) = @_;

    printlabel $name, "command: $cmd";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;
    foreach my $p (split /[;:]/o, $ENV{'PATH'})
    {
        if (-x "$p/$cmd")
        {
            $r_config->{$name} = "$p/$cmd";
            last;
        }
    }
    printyesno $name, $r_config->{$name};
}

sub
check_ifoption
{
    my ($ifcount, $type, $name, $opt, $r_clist, $r_config) = @_;

    printlabel $name, "$type ($ifcount): $opt";

    loadoptions ();

    my $trc = 'F';

    my $found = 'F';
    if ($optionsloaded && defined ($optionshash{$opt})) {
      $found = 'T';
      $trc = lc $optionshash{$opt};
      print LOGFH "##  found: $opt => $trc\n";
      if ($trc eq 't') { $trc = 'T'; }
      if ($trc eq 'enable') { $trc = 'T'; }
      if ($trc eq 'f') { $trc = 'F'; }
      if ($trc eq 'disable') { $trc = 'F'; }
      if ($trc eq 'true') { $trc = 'T'; }
      if ($trc eq 'false') { $trc = 'F'; }
    }

    if ($trc eq 'T') { $trc = 1; }
    if ($trc eq 'F') { $trc = 0; }

    if ($type eq 'ifnotoption') {
      $trc = $trc == 0 ? 1 : 0;
    }

    if (! $optionsloaded) {
      $trc = 0;
      printyesno_actual $name, "no options file";
    } elsif ($found eq 'F') {
      $trc = 0;
      printyesno_actual $name, "option not found";
    } else {
      printyesno $name, $trc;
    }
    return $trc;
}

sub
check_if
{
    my ($iflabel, $ifcount, $ifline, $r_clist, $r_config) = @_;

    my $name = "_if_$iflabel";
    my $trc = 0;
    # make parseable
    $ifline =~ s/!/ ! /og;
    $ifline =~ s/\(/ ( /og;
    $ifline =~ s/\)/ ) /og;
    $ifline =~ s/&&/ \&\& /og;
    $ifline =~ s/\|\|/ || /og;
    $ifline =~ s/ and / && /og;
    $ifline =~ s/ or / || /og;
    $ifline =~ s/ not / ! /og;
    $ifline =~ s/ +/ /og;
    $ifline =~ s/^ *//og;
    $ifline =~ s/ *$//og;

    printlabel $name, "if ($ifcount): $iflabel";
    my $nline = '';
    print LOGFH "## ifline: $ifline\n";
    foreach my $token (split (/\s/, $ifline)) {
      if ($token eq '(' || $token eq ')' || $token eq '&&' ||
          $token eq '||' || $token eq '!') {
        $nline .= $token . ' ';
      } else {
        print LOGFH "## value of $token: " . $r_config->{$token} . "\n";
        $nline .= $r_config->{$token} ne '0' ? 1 : 0;
        $nline .= ' ';
      }
    }
    print LOGFH "## nline: $nline\n";
    $trc = eval $nline;
    printyesno $name, $trc;
    return $trc;
}

sub
check_set
{
    my ($name, $type, $val, $r_clist, $r_config) = @_;

    my $tnm = $name;
    $tnm =~ s/^_setint_//;
    $tnm =~ s/^_setstr_//;
    printlabel $name, "${type}: $tnm";

    if ($type eq 'set') {
      if (defined ($r_config->{$name})) {
        setlist $r_clist, $name;
        $r_config->{$name} = $val;
        printyesno $name, $r_config->{$name};
      } else {
        printyesno_actual $name, 'no such variable';
      }
    } elsif ($type eq 'setint') {
      setlist $r_clist, $name;
      $r_config->{$name} = $val;
      printyesno_actual $name, $r_config->{$name};
    } else {
      setlist $r_clist, $name;
      $r_config->{$name} = $val;
      printyesno_actual $name, $r_config->{$name};
    }
}

sub
check_option
{
    my ($name, $onm, $def, $r_clist, $r_config) = @_;

    printlabel $name, "option: $onm";

    loadoptions ();
    my $oval = $def;

    if ($optionsloaded && defined ($optionshash{$onm})) {
      $oval = $optionshash{$onm};
      print LOGFH "##  found: $onm => $oval\n";
    }

    setlist $r_clist, $name;
    $r_config->{$name} = $oval;
    printyesno_actual $name, $r_config->{$name};
}

sub
check_quotactl_pos
{
  my ($name, $r_clist, $r_config) = @_;

  printlabel $name, "quotactl position";
  if (checkcache ($name, $r_config) == 0) {
    return;
  }

  setlist $r_clist, $name;

  if ($r_config->{'_lib_quotactl'} eq '0') {
    $r_config->{$name} = '0';
    printyesno_val $name, $r_config->{$name};
    return;
  }

  my $hdr = '';
  foreach my $hnm ('_sys_quota', '_hdr_ufs_ufs_quota', '_hdr_ufs_quota',
      '_hdr_linux_quota') {
    if ($r_config->{$hnm} ne '0') {
      $hdr = $r_config->{$hnm};
      last;
    }
  }
  my $uhdr = $r_config->{'_hdr_unistd'};

  my $code = '';
  if ($hdr ne '') {
    $code .= "#include <${hdr}>\n";
    if ($uhdr ne '0') {
      $code .= "#include <${uhdr}>\n";
    }
    my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
    if ($rc == 0) {
       $cmd = "egrep -l 'quotactl *\\( *int[^,]*, *(const *)?char' $name.out >>$LOG 2>&1";
       print LOGFH "##  quotactl_pos: $cmd\n";
       $rc = system ($cmd);
       if ($rc == 0) {
         $r_config->{$name} = 2;
         printyesno_val $name, $r_config->{$name};
         return;
       }

       $cmd = "egrep -l 'quotactl *\\( *(const *)?char[^,]*, *int' $name.out >>$LOG 2>&1";
       print LOGFH "##  quotactl_pos: $cmd\n";
       $rc = system ($cmd);
       if ($rc == 0) {
         $r_config->{$name} = 1;
         printyesno_val $name, $r_config->{$name};
         return;
       }
    }
  }

  $r_config->{$name} = 0;
  printyesno_val $name, $r_config->{$name};
}

sub
check_rquota_xdr
{
  my ($name, $r_clist, $r_config) = @_;

  printlabel $name, "rquota xdr";
  if (checkcache ($name, $r_config) == 0) {
    return;
  }

  setlist $r_clist, $name;

  if ($r_config->{'_hdr_rpcsvc_rquota'} eq '0') {
    $r_config->{$name} = '0';
    printyesno_val $name, $r_config->{$name};
    return;
  }

  my $code = '';
  my $hdr = $r_config->{'_hdr_rpcsvc_rquota'};
  $code .= "#include <$hdr>\n";
  my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
  if ($rc == 0) {
    $cmd = "sed '1,/struct *rquota *{/d' $name.out |
        egrep '[	 ]*rq_bhardlimit[	 ]*;' >>$LOG 2>&1";
    print LOGFH "##  rquota_xdr: $cmd\n";
    $rc = system ($cmd);
    if ($rc == 0) {
      my $rval = `sed '1,/struct *rquota *{/d' $name.out |
          egrep '[	 ]*rq_bhardlimit[	 ]*;'`;
      chomp $rval;
      $rval =~ s/[	 ][	 ]*rq_bhardlimit.*//;
      $rval =~ s/^[	 ]*//;
      $r_config->{$name} = "xdr_" . ${rval};
      printyesno_val $name, $r_config->{$name};
      return;
    }
  }

  $r_config->{$name} = 0;
  printyesno_val $name, $r_config->{$name};
}

sub
check_gqa_uid_xdr
{
  my ($name, $r_clist, $r_config) = @_;

  printlabel $name, "gqa_uid xdr";
  if (checkcache ($name, $r_config) == 0) {
    return;
  }

  setlist $r_clist, $name;

  if ($r_config->{'_hdr_rpcsvc_rquota'} eq '0') {
    $r_config->{$name} = '0';
    printyesno_val $name, $r_config->{$name};
    return;
  }

  my $code = '';
  my $hdr = $r_config->{'_hdr_rpcsvc_rquota'};
  $code .= "#include <$hdr>\n";
  my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
  if ($rc == 0) {
    $cmd = "sed '1,/struct *getquota_args *{/d' $name.out |
        egrep '[	 ]*gqa_uid[	 ]*;' >>$LOG 2>&1";
    print LOGFH "##  gqa_uid_xdr: $cmd\n";
    $rc = system ($cmd);
    if ($rc == 0) {
      my $rval = `sed '1,/struct *getquota_args *{/d' $name.out |
          egrep '[	 ]*gqa_uid[	 ]*;'`;
      chomp $rval;
      $rval =~ s/[	 ][	 ]*gqa_uid.*//;
      $rval =~ s/^[	 ]*//;
      $r_config->{$name} = "xdr_" . ${rval};
      printyesno_val $name, $r_config->{$name};
      return;
    }
  }

  $r_config->{$name} = 0;
  printyesno_val $name, $r_config->{$name};
}

sub
check_getfsstat_type
{
  my ($name, $r_clist, $r_config) = @_;

  printlabel $name, "getfsstat type";
  if (checkcache ($name, $r_config) == 0) {
    return;
  }

  setlist $r_clist, $name;

  if ($r_config->{'_lib_getfsstat'} eq '0') {
    $r_config->{$name} = '0';
    printyesno_val $name, $r_config->{$name};
    return;
  }

  if ($r_config->{'_sys_mount'} eq '0') {
    $r_config->{$name} = '0';
    printyesno_val $name, $r_config->{$name};
    return;
  }

  my $code = '';
  my $hdr = $r_config->{'_sys_mount'};
  $code .= "#include <$hdr>\n";
  my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
  if ($rc == 0) {
    $cmd = "egrep -l 'getfsstat[	 ]*\\(' $name.out >>$LOG 2>&1";
    $rc = system ($cmd);
    print LOGFH "##  getfsstat_type: $cmd $rc\n";
    if ($rc == 0) {
      my $rval = `egrep 'getfsstat[	 ]*\\(' $name.out`;
      print LOGFH "##  getfsstat_type: $rval\n";
      chomp $rval;
      $rval =~ s/^[^,]*,[	 ]*//;
      $rval =~ s/,.*$//;
      $rval =~ s/  *[_a-z0-9]*$//;  # strip any trailing var name
      print LOGFH "##  getfsstat_type: $rval\n";
      $r_config->{$name} = ${rval};
      printyesno_val $name, $r_config->{$name};
      return;
    }
  }

  $r_config->{$name} = 0;
  printyesno_val $name, $r_config->{$name};
}

sub
check_include_conflict
{
    my ($name, $h1, $h2, $r_clist, $r_config) = @_;

    my $i1 = $h1;
    $i1 =~ s,/,_,go;
    $i1 =~ s,\.h$,,o;
    $i1 =~ s,:,,go;
    if ($i1 =~ m#^sys#o) {
      $i1 = "_${i1}";
    } else {
      $i1 = "_hdr_${i1}";
    }
    my $i2 = $h2;
    $i2 =~ s,/,_,go;
    $i2 =~ s,\.h$,,o;
    $i2 =~ s,:,,go;
    if ($i2 =~ m#^sys#o) {
      $i2 = "_${i2}";
    } else {
      $i2 = "_hdr_${i2}";
    }

    $name = "${name}_${i1}_${i2}";
    printlabel $name, "header: include both $h1 & $h2";

    if (defined ($r_config->{${i1}}) &&
        $r_config->{${i1}} ne '0' &&
        defined ($r_config->{${i2}}) &&
        $r_config->{${i2}} ne '0')
    {
        if (checkcache ($name, $r_config) == 0)
        {
            return;
        }

        my $code = <<"_HERE_";
#include <$h1>
#include <$h2>
main () { return 0; }
_HERE_
        do_chk_compile ($name, $code, 'std', $r_clist, $r_config);
    } else {
        setlist $r_clist, $name;
        $r_config->{$name} = 1;
        printyesno $name, $r_config->{$name};
    }
}

sub
check_npt
{
    my ($name, $proto, $req, $r_clist, $r_config) = @_;

    printlabel $name, "need prototype: $proto";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    if (defined ($r_config->{$req}) && $r_config->{$req} eq '0')
    {
      $r_config->{$name} = 0;
      setlist $r_clist, $name;
      printyesno $name, $r_config->{$name};
      return;
    }

    my $code = <<"_HERE_";
CPP_EXTERNS_BEG
struct _TEST_struct { int _TEST_member; };
extern struct _TEST_struct* $proto _((struct _TEST_struct*));
CPP_EXTERNS_END
_HERE_
    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
}

sub
check_type
{
    my ($name, $type, $r_clist, $r_config) = @_;

    $type =~ s/star/*/og;
    printlabel $name, "type: $type";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    my $code = <<"_HERE_";
struct xxx { $type mem; };
static struct xxx v;
struct xxx* f() { return &v; }
main () { struct xxx *tmp; tmp = f(); return (0); }
_HERE_
    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
}

sub
check_defined
{
    my ($name, $def, $r_clist, $r_config) = @_;

    printlabel $name, "defined: $def";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    my $code = <<"_HERE_";
main () {
#ifdef ${def}
return (0);
#else
return (1);
#endif
}
_HERE_
    my $val = 0;
    my $rc = _chk_run ($name, $code, \$val, $r_clist, $r_config, {});
    $r_config->{$name} = $rc == 0 ? 1 : 0;
    printyesno $name, $r_config->{$name};
}

sub
check_param_void_star
{
    my ($name, $r_clist, $r_config) = @_;

    printlabel $name, "parameter: void *";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    my $code = <<"_HERE_";
char *
tparamvs (ptr)
  void *ptr;
{
  ptr = (void *) NULL;
  return (char *) ptr;
}
_HERE_
    do_chk_compile ($name, $code, 'all', $r_clist, $r_config);
}

sub
check_lib
{
    my ($name, $func, $r_clist, $r_config, $r_a) = @_;

    setlist $r_clist, $name;
    my $val = $r_a->{'otherlibs'} || '';

    $rfunc = $func;
    $rfunc =~ s/_dollar_/\$/g;
    if ($val ne '')
    {
        printlabel $name, "function: $rfunc [$val]";
    }
    else
    {
        printlabel $name, "function: $rfunc";
        if (checkcache ($name, $r_config) == 0)
        {
            return;
        }
    }

    $r_config->{$name} = 0;
    # unfortunately, this does not work if the function
    # is not declared.
    my $code = <<"_HERE_";
typedef int (*_TEST_fun_)();
static _TEST_fun_ i=(_TEST_fun_) $rfunc;
main () {  i(); return (i==0); }
_HERE_

    my %a = (
         'incheaders' => 'all',
         'otherlibs' => $val,
         );
    my $rc = _chk_link ($name, $code, $r_clist, $r_config, \%a);
    my $tag = '';
    if ($rc == 0)
    {
      $r_config->{$name} = 1;
      if ($a{'dlibs'} ne '')
      {
          $tag = " with $a{'dlibs'}";
          $r_config->{"lib_$name"} = $a{'dlibs'};
          setlist $r_clist, "lib_$name";
      }
    }
    printyesno $name, $r_config->{$name}, $tag;
}

sub
check_class
{
    my ($name, $class, $r_clist, $r_config, $r_a) = @_;

    setlist $r_clist, $name;
    my $val = $r_a->{'otherlibs'} || '';

    if ($val ne '')
    {
        printlabel $name, "class: $class [$val]";
    }
    else
    {
        printlabel $name, "class: $class";
        if (checkcache ($name, $r_config) == 0)
        {
            return;
        }
    }

    $r_config->{$name} = 0;
    my $code = <<"_HERE_";
main () { $class testclass; }
_HERE_

    my %a = (
         'incheaders' => 'all',
         'otherlibs' => $val,
         );
    my $rc = _chk_link ($name, $code, $r_clist, $r_config, \%a);
    my $tag = '';
    if ($rc == 0)
    {
      $r_config->{$name} = 1;
      if ($a{'dlibs'} ne '')
      {
          $tag = " with $a{'dlibs'}";
          $r_config->{"lib_$name"} = $a{'dlibs'};
          setlist $r_clist, "lib_$name";
      }
    }
    printyesno $name, $r_config->{$name}, $tag;
}

sub
check_setmntent_args
{
    my ($name, $r_clist, $r_config) = @_;

    printlabel $name, "setmntent # arguments";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;

    if ($r_config->{'_lib_setmntent'} eq '0')
    {
        printyesno_val $name, $r_config->{$name};
        return;
    }
    my $code = <<"_HERE_";
main () { setmntent ("/etc/mnttab"); }
_HERE_
    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', 'otherlibs' => undef, });
    if ($rc == 0)
    {
        $r_config->{$name} = 2;
        printyesno_val $name, $r_config->{$name};
        return;
    }

    $code = <<"_HERE_";
main () { setmntent ("/etc/mnttab", "r"); }
_HERE_
    $rc = _chk_link ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', 'otherlibs' => undef, });
    if ($rc == 0)
    {
        $r_config->{$name} = 3;
    }
    printyesno_val $name, $r_config->{$name};
}

sub
check_statfs_args
{
    my ($name, $r_clist, $r_config) = @_;

    printlabel $name, "statfs # arguments";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;

    if ($r_config->{'_lib_statfs'} eq '0')
    {
        printyesno_val $name, $r_config->{$name};
        return;
    }

    my $code = <<"_HERE_";
main () {
    struct statfs statBuf; char *name; name = "/";
    statfs (name, &statBuf);
}
_HERE_
    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', 'otherlibs' => undef, });
    if ($rc == 0)
    {
        $r_config->{$name} = 2;
        printyesno_val $name, $r_config->{$name};
        return;
    }

    $code = <<"_HERE_";
main () {
    struct statfs statBuf; char *name; name = "/";
    statfs (name, &statBuf, sizeof (statBuf));
}
_HERE_
    $rc = _chk_link ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', 'otherlibs' => undef, });
    if ($rc == 0)
    {
        $r_config->{$name} = 3;
        printyesno_val $name, $r_config->{$name};
        return;
    }

    $code = <<"_HERE_";
main () {
    struct statfs statBuf; char *name; name = "/";
    statfs (name, &statBuf, sizeof (statBuf), 0);
}
_HERE_
    $rc = _chk_link ($name, $code, $r_clist, $r_config,
        { 'incheaders' => 'all', 'otherlibs' => undef, });
    if ($rc == 0)
    {
        $r_config->{$name} = 4;
    }
    printyesno_val $name, $r_config->{$name};
}

sub
check_size
{
    my ($name, $type, $r_clist, $r_config) = @_;

    printlabel $name, "sizeof: $type";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;
    my $code = <<"_HERE_";
main () {
	printf("%u\\n", sizeof($type));
    return (0);
    }
_HERE_
    my $val = 0;
    my $rc = _chk_run ($name, $code, \$val, $r_clist, $r_config, {});
    if ($rc == 0)
    {
        $r_config->{$name} = $val;
    }
    printyesno_val $name, $r_config->{$name};
}

sub
check_member
{
    my ($name, $struct, $member, $r_clist, $r_config) = @_;

    printlabel $name, "exists: $struct.$member";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;
    my $code = <<"_HERE_";
main () { $struct s; int i; i = sizeof (s.$member); }
_HERE_
    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
            { 'incheaders' => 'all', });
    if ($rc == 0)
    {
        $r_config->{$name} = 1;
    }
    printyesno $name, $r_config->{$name};
}

sub
check_int_declare
{
    my ($name, $function, $r_clist, $r_config) = @_;

    printlabel $name, "declared: $function";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;
    my $code = <<"_HERE_";
    main () { int x; x = $function; }
_HERE_
    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
            { 'incheaders' => 'all', });
    if ($rc == 0)
    {
        $r_config->{$name} = 1;
    }
    printyesno $name, $r_config->{$name};
}

sub
check_ptr_declare
{
    my ($name, $function, $r_clist, $r_config) = @_;

    printlabel $name, "declared: $function";
    if (checkcache ($name, $r_config) == 0)
    {
        return;
    }

    setlist $r_clist, $name;
    $r_config->{$name} = 0;
    my $code = <<"_HERE_";
main () { void *x; x = $function; }
_HERE_
    my $rc = _chk_compile ($name, $code, $r_clist, $r_config,
            { 'incheaders' => 'all', });
    if ($rc == 0)
    {
        $r_config->{$name} = 1;
    }
    printyesno $name, $r_config->{$name};
}

sub
check_standard
{
  my ($r_clist, $r_config) = @_;
  # FreeBSD has buggy headers, requires sys/param.h as a required include.
  # always check for these headers.
  my @headlist1 = (
      [ "_hdr_stdio", "stdio.h", ],
      [ "_hdr_stdlib", "stdlib.h", ],
      [ "_sys_types", "sys/types.h", ],
      [ "_sys_param", "sys/param.h", ],
      );

  foreach my $r_arr (@headlist1)
  {
      check_header ($$r_arr[0], $$r_arr[1], $r_clist, $r_config,
              { 'reqhdr' => [], });
  }
  check_keyword ('_key_void', 'void', $r_clist, $r_config);
  check_keyword ('_key_const', 'const', $r_clist, $r_config);
  check_param_void_star ('_param_void_star', $r_clist, $r_config);
  check_proto ('_proto_stdc', $r_clist, $r_config);
}


sub
create_output
{
  my ($r_clist, $r_config, $include) = @_;

  if ($CONFH eq 'none') { return; }

  open (CCOFH, ">$CONFH");
  print CCOFH <<'_HERE_';
#ifndef __INC_CONFIG_H
#define __INC_CONFIG_H 1

_HERE_

  foreach my $val (@{$r_clist->{'list'}})
  {
    if ($val =~ m#^lib__lib_#o) {
      next;
    }
    my $tval = 0;
    if ($r_config->{$val} ne "0") {
        $tval = 1;
    }
    if ($val =~ m#^_setint_#o) {
      $tnm = $val;
      $tnm =~ s/^_setint_//;
      print CCOFH "#define $tnm " . $r_config->{$val} . "\n";
    } elsif ($val =~ m#^_setstr_#o || $val =~ m#^_opt_#o) {
      $tnm = $val;
      $tnm =~ s/^_setstr_//;
      $tnm =~ s/^_opt_//;
      print CCOFH "#define $tnm \"" . $r_config->{$val} . "\"\n";
    } elsif ($val =~ m#^(_hdr|_sys|_command)#o) {
      print CCOFH "#define $val $tval\n";
    } else {
      print CCOFH "#define $val " . $r_config->{$val} . "\n";
    }
  }

  # standard tail -- always needed; non specific
  print CCOFH <<'_HERE_';

#if ! _key_void
# define void int
#endif
#if ! _key_void || ! _param_void_star
  typedef char *_pvoid;
#else
  typedef void *_pvoid;
#endif
#if ! _key_const
# define const
#endif

#ifndef _
# if _proto_stdc
#  define _(args) args
# else
#  define _(args) ()
# endif
#endif

_HERE_

  print CCOFH $include;

  print CCOFH <<'_HERE_';

#endif /* __INC_CONFIG_H */
_HERE_
  close CCOFH;
}

sub
main_process
{
    my ($configfile) = @_;
    my (%clist, %config);

    $clist{'list'} = ();
    $clist{'hash'} = {};
    $config{'reqlibs'} = {};
    $config{'reqlibs_list'} = ();

    if (-f $CACHEFILE)
    {
      open (MKCC, "<$CACHEFILE");
      while (my $line = <MKCC>)
      {
        chomp $line;
        if ($line =~ m/^di_c_(.*)='(.*)'/o)
        {
          my $name = $1;
          my $val = $2;
          $config{$name} = $val;
          $clist{'hash'}->{$name} = 1;
        }
      }
      close (MKCC);
      open (MKCV, "<$VARSFILE");
      while (my $line = <MKCV>)
      {
        chomp $line;
        push @{$clist{'list'}}, $line;
      }
      close (MKCV);
    }


    my $tconfigfile = $configfile;
    if ($configfile !~ m#^/#) {
      $tconfigfile = "../$configfile";
    }
    if (! open (DATAIN, "<$tconfigfile"))
    {
        print STDOUT "$configfile: $!\n";
        exit 1;
    }

    my $linenumber = 0;
    my $inheaders = 1;
    my $ininclude = 0;
    my @doproclist;
    my $inproc = 0;
    my $doproc = 1;
    my $include = '';
    my $tline = '';
    my $ifstmtcount = 0;
    while (my $line = <DATAIN>)
    {
      chomp $line;
      ++$linenumber;

      if ($ininclude == 0 && ($line =~ /^#/o || $line eq ''))
      {
          next;
      }

      if ($tline ne '') {
        $line = $tline . ' ' . $line;
        $tline = '';
      }
      if ($line =~ /[^\\]\\$/o) {
        $tline = $line;
        $tline =~ s/\\$//o;
        next;
      }

      if ($ininclude == 1 && $line =~ m#^\s*endinclude$#o)
      {
          print LOGFH "end include\n";
          $ininclude = 0;
          next;
      }
      elsif ($ininclude == 1)
      {
          $line =~ s,\\(.),$1,g;
          $include .= $line . "\n";
          next;
      }

      if ($inheaders && $line !~ m#^\s*(hdr|sys)#o)
      {
          $inheaders = 0;
      }

      print LOGFH "#### ${linenumber}: ${line}\n";

      if ($line =~ m#^\s*else#o)
      {
          $doproc = $doproc == 0 ? 1 : 0;
          $iflevels =~ s/.\d+\s$//;
          $iflevels .= "-$ifstmtcount ";
      }
      elsif ($line =~ m#^\s*endif#o)
      {
        if ($#doproclist >= 0) {
          print LOGFH "## endif: doproclist: " . join (' ', @doproclist) . "\n";
          $doproc = shift @doproclist;
          print LOGFH "## endif: doproclist now : " . join (' ', @doproclist) . "\n";
          print LOGFH "## endif: doproc: $doproc\n";
          $iflevels =~ s/.\d+\s$//;
        } else {
          $doproc = 1;
          $iflevels = '';
        }
      }

      if ($doproc == 1) {
        if ($line =~ m#^\s*output\s+([^\s]+)#o)
        {
            if ($inproc == 1) {
              create_output (\%clist, \%config, $include);
              $CONFH = 'none';
              $include = '';
            }

            print "output-file: $1\n";
            my $tconfh = $1;
            if ($tconfh =~ m#^/#o) {
              $CONFH = $tconfh;
            } else {
              $CONFH = "../$tconfh";
            }
            print LOGFH "config file: $CONFH\n";
            $inproc = 1;
        }
        elsif ($line =~ m#^\s*option\-file\s+([^\s]+)#o)
        {
            print "option-file: $1\n";
            my $tfile = $1;
            if ($tfile =~ m#^/#o) {
              $OPTIONFILE = $tfile;
            } else {
              $OPTIONFILE = "../$tfile";
            }
            print LOGFH "options file: $OPTIONFILE\n";
        }
        elsif ($line =~ m#^\s*standard#o)
        {
            check_standard (\%clist, \%config);
        }
        elsif ($line =~ m#^\s*loadunit#o)
        {
            ;
        }
        elsif ($line =~ m#^\s*setmntent_args#o)
        {
            check_setmntent_args ('_setmntent_args', \%clist, \%config);
        }
        elsif ($line =~ m#^\s*statfs_args#o)
        {
            check_statfs_args ('_statfs_args', \%clist, \%config);
        }
        elsif ($line =~ m#^\s*include_conflict\s+([^\s]+)\s+([^\s]+)#o)
        {
            my $i1 = $1;
            my $i2 = $2;
            check_include_conflict ('_inc_conflict', $i1, $i2, \%clist, \%config);
        }
        elsif ($line =~ m#^\s*quotactl_pos$#o)
        {
            check_quotactl_pos ('_quotactl_pos', \%clist, \%config);
        }
        elsif ($line =~ m#^\s*rquota_xdr$#o)
        {
            check_rquota_xdr ('_rquota_xdr', \%clist, \%config);
        }
        elsif ($line =~ m#^\s*gqa_uid_xdr$#o)
        {
            check_gqa_uid_xdr ('_gqa_uid_xdr', \%clist, \%config);
        }
        elsif ($line =~ m#^\s*getfsstat_type$#o)
        {
            check_getfsstat_type ('_getfsstat_type', \%clist, \%config);
        }
        elsif ($line =~ m#^\s*include$#o)
        {
            print LOGFH "start include\n";
            $ininclude = 1;
        }
        elsif ($line =~ m#^\s*(hdr|sys)\s+([^\s]+)\s*(.*)#o)
        {
            my $typ = $1;
            my $hdr = $2;
            my $reqhdr = $3;
            my $nm = "_${typ}_";
            # create the standard header name for config.h
            $nm .= $hdr;
            $nm =~ s,/,_,go;
            $nm =~ s,\.h$,,o;
            $nm =~ s,:,_,go;
            if ($typ eq 'sys') {
              $hdr = 'sys/' . $hdr;
            }
            $reqhdr =~ s/^\s*//o;
            $reqhdr =~ s/\s*$//o;
            my @oh = split (/\s+/, $reqhdr);
            check_header ($nm, $hdr, \%clist, \%config, { 'reqhdr' => \@oh, });
        }
        elsif ($line =~ m#^\s*const\s+([^\s]+)\s*(.*)#o)
        {
            my $tnm = $1;
            my $reqhdr = $2;
            my $nm = "_const_" . $tnm;
            $reqhdr =~ s/^\s*//o;
            $reqhdr =~ s/\s*$//o;
            my @oh = split (/\s+/, $reqhdr);
            check_constant ($nm, $tnm, \%clist, \%config,
                { 'reqhdr' => \@oh, });
        }
        elsif ($line =~ m#^\s*command\s+(.*)#o)
        {
            my $cmd = $1;
            my $nm = "_command_" . $cmd;
            check_command ($nm, $cmd, \%clist, \%config);
        }
        elsif ($line =~ m#^\s*(if(not)?option)\s+([^\s]+)#o)
        {
            my $type = $1;
            my $opt = $3;
            my $nm = "_${type}_${opt}";
            ++$ifstmtcount;
            my $rc = check_ifoption ($ifstmtcount, $type, $nm, $opt, \%clist, \%config);
            $iflevels .= "+$ifstmtcount ";
            unshift @doproclist, $doproc;
            $doproc = $rc;
            print LOGFH "## ifoption: doproclist: " . join (' ', @doproclist) . "\n";
            print LOGFH "## ifoption: doproc: $doproc\n";
        }
        elsif ($line =~ m#^\s*if\s+([^\s]+)\s+(.*)#o)
        {
            my $iflabel = $1;
            my $ifline = $2;
            ++$ifstmtcount;
            print LOGFH "## if: label: $iflabel count $ifstmtcount line: $ifline\n";
            my $rc = check_if ($iflabel, $ifstmtcount, $ifline, \%clist, \%config);
            $iflevels .= "+$ifstmtcount ";
            unshift @doproclist, $doproc;
            $doproc = $rc;
            print LOGFH "## if: doproclist: " . join (' ', @doproclist) . "\n";
            print LOGFH "## if: doproc: $doproc\n";
        }
        elsif ($line =~ m#^\s*(endif|else)#o)
        {
            ;
        }
        elsif ($line =~ m#^\s*(set(int|str)?)\s+([^\s]+)\s*(.*)#o)
        {
            my $type = $1;
            my $nm = $3;
            if ($type eq 'setint' || $type eq 'setstr') {
              $nm = "_${type}_$3";
            }
            my $val = $4;
            check_set ($nm, $type, $val, \%clist, \%config);
        }
        elsif ($line =~ m#^\s*option\s+([^\s]+)\s*(.*)#o)
        {
            my $nm = "_opt_$1";
            my $onm = $1;
            my $def = $2;
            check_option ($nm, $onm, $def, \%clist, \%config);
        }
        elsif ($line =~ m#^\s*npt\s+([^\s]*)\s*(.*)#o)
        {
            my $func = $1;
            my $req = $2;
            my $nm = "_npt_" . $func;
            check_npt ($nm, $func, $req, \%clist, \%config);
        }
        elsif ($line =~ m#^\s*key\s+(.*)#o)
        {
            my $tnm = $1;
            my $nm = "_key_" . $tnm;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_keyword ($nm, $tnm, \%clist, \%config);
            }
        }
        elsif ($line =~ m#^\s*class\s+([^\s]+)\s*(.*)?#o)
        {
            my $class = $1;
            my $libs = $2 || '';
            my $nm = "_class_" . $class;
            $nm =~ s,:,_,go;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_class ($nm, $class, \%clist, \%config,
                       { 'otherlibs' => $libs, });
            }
        }
        elsif ($line =~ m#^\s*typ\s+(.*)#o)
        {
            my $tnm = $1;
            my $nm = "_typ_" . $tnm;
            $nm =~ s, ,_,go;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_type ($nm, $tnm, \%clist, \%config);
            }
        }
        elsif ($line =~ m#^\s*define\s+(.*)#o)
        {
            my $tnm = $1;
            my $nm = "_define_" . $tnm;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_defined ($nm, $tnm, \%clist, \%config);
            }
        }
        elsif ($line =~ m#^\s*lib\s+([^\s]+)\s*(.*)?#o)
        {
            my $func = $1;
            my $libs = $2 || '';
            my $nm = "_lib_" . $func;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_lib ($nm, $func, \%clist, \%config,
                       { 'otherlibs' => $libs, });
            }
        }
        elsif ($line =~ m#^\s*dcl\s+([^\s]*)\s+(.*)#o)
        {
            my $type = $1;
            my $var = $2;
            my $nm = "_dcl_" . $var;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                if ($type eq 'int')
                {
                    check_int_declare ($nm, $var, \%clist, \%config);
                }
                elsif ($type eq 'ptr')
                {
                    check_ptr_declare ($nm, $var, \%clist, \%config);
                }
            }
        }
        elsif ($line =~ m#^\s*member\s+(.*)\s+([^\s]+)#o)
        {
            my $struct = $1;
            my $member = $2;
            my $nm = "_mem_" . $struct . '_' . $member;
            $nm =~ s/ /_/go;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_member ($nm, $struct, $member, \%clist, \%config);
            }
        }
        elsif ($line =~ m#^\s*size\s+(.*)#o)
        {
            my $typ = $1;
            $typ =~ s/\s*$//o;
            my $nm = "_siz_" . $typ;
            $nm =~ s, ,_,go;
            if (! defined ($config{$nm}) ||
                $config{$nm} eq '0')
            {
                check_size ($nm, $typ, \%clist, \%config);
            }
        }
        else
        {
            print LOGFH "unknown command: $line\n";
            print STDOUT "unknown command: $line\n";
        }
      }
    }

    savecache (\%clist, \%config);
    create_output (\%clist, \%config, $include);

    # This is here for os/2 and other systems w/no usable bourne shell.
    # it is not up to date, as it will output libraries that are not
    # needed by the final configuration.
    open (RLIBFH, ">$REQLIB");
    my $r_list = $config{'reqlibs_list'};
    print RLIBFH join (' ', @{$r_list}) . "\n";
    close RLIBFH;
}

sub
usage
{
  print STDOUT "Usage: $0 [-c <cache-file>] [-L <log-file>]\n";
  print STDOUT "       [-o <option-file>] [-C] <config-file>\n";
  print STDOUT "  -C : clear cache-file\n";
  print STDOUT "defaults:\n";
  print STDOUT "  <cache-file> : mkconfig.cache\n";
  print STDOUT "  <log-file>   : mkconfig.log\n";
  print STDOUT "  <option-file>: options.dat\n";
}

# main

my $clearcache = 0;
while ($#ARGV > 0)
{
  if ($ARGV[0] eq "-C")
  {
      shift @ARGV;
      $clearcache = 1;
  }
  if ($ARGV[0] eq "-c")
  {
      shift @ARGV;
      $CACHEFILE = $ARGV[0];
      shift @ARGV;
  }
  if ($ARGV[0] eq "-L")
  {
      shift @ARGV;
      $LOG = $ARGV[0];
      shift @ARGV;
  }
  if ($ARGV[0] eq "-o")
  {
      shift @ARGV;
      $OPTIONFILE = $ARGV[0];
      shift @ARGV;
  }
}

my $configfile = $ARGV[0];
if (! defined ($configfile) || ! -f $configfile)
{
  usage;
  exit 1;
}
if (-d $TMP && $TMP ne "_tmp_mkconfig")
{
  usage;
  exit 1;
}

$LOG = "../$LOG";
$REQLIB = "../$REQLIB";
$CACHEFILE = "../$CACHEFILE";
$VARSFILE = "../$VARSFILE";
$OPTIONFILE = "../$OPTIONFILE";

if (-d $TMP) { system ("rm -rf $TMP"); }
mkdir $TMP, 0777;
chdir $TMP;

if ($clearcache)
{
    unlink $CACHEFILE;
    unlink $VARSFILE;
}

print STDOUT "$0 using $configfile\n";
unlink $LOG;
open (LOGFH, ">>$LOG");
$ENV{'CFLAGS'} = $ENV{'CFLAGS'};
print LOGFH "CC: $ENV{'CC'}\n";
print LOGFH "CFLAGS: $ENV{'CFLAGS'}\n";
print LOGFH "CPPFLAGS: $ENV{'CPPFLAGS'}\n";
print LOGFH "LDFLAGS: $ENV{'LDFLAGS'}\n";
print LOGFH "LIBS: $ENV{'LIBS'}\n";

main_process $configfile;

close LOGFH;

chdir "..";
if (-d $TMP) { system ("rm -rf $TMP"); }
exit 0;
